@@ -244,17 +244,18 @@ pub(crate) async fn pip_install(
     // Resolve the requirements from the provided sources.
     let requirements = {
         // Convert from unnamed to named requirements.
-        let mut requirements = NamedRequirementsResolver::new(requirements)
-            .with_reporter(ResolverReporter::from(printer))
-            .resolve(&resolve_dispatch, &client)
-            .await?;
+        let mut requirements =
+            NamedRequirementsResolver::new(requirements, &resolve_dispatch, &client, &index)
+                .with_reporter(ResolverReporter::from(printer))
+                .resolve()
+                .await?;
 
         // Resolve any source trees into requirements.
         if !source_trees.is_empty() {
             requirements.extend(
-                SourceTreeResolver::new(source_trees, extras)
+                SourceTreeResolver::new(source_trees, extras, &resolve_dispatch, &client, &index)
                     .with_reporter(ResolverReporter::from(printer))
-                    .resolve(&resolve_dispatch, &client)
+                    .resolve()
                     .await?,
             );
         }
@@ -539,10 +540,18 @@ async fn resolve(
         .collect();
 
     // Determine any lookahead requirements.
-    let lookaheads = LookaheadResolver::new(&requirements, &constraints, &overrides, &editables)
-        .with_reporter(ResolverReporter::from(printer))
-        .resolve(build_dispatch, markers, client)
-        .await?;
+    let lookaheads = LookaheadResolver::new(
+        &requirements,
+        &constraints,
+        &overrides,
+        &editables,
+        build_dispatch,
+        client,
+        index,
+    )
+    .with_reporter(ResolverReporter::from(printer))
+    .resolve(markers)
+    .await?;
 
     // Create a manifest of the requirements.
     let manifest = Manifest::new(
