@@ -16,13 +16,14 @@ use distribution_types::{FlatIndexLocation, IndexLocations, IndexUrl};
 use uv_auth::KeyringProvider;
 use uv_cache::{Cache, CacheArgs, Refresh};
 use uv_client::Connectivity;
-use uv_installer::{NoBinary, Reinstall};
 use uv_interpreter::PythonVersion;
 use uv_normalize::{ExtraName, PackageName};
 use uv_requirements::{ExtrasSpecification, RequirementsSource};
 use uv_resolver::{AnnotationStyle, DependencyMode, PreReleaseMode, ResolutionMode};
+use uv_types::NoBinary;
 use uv_types::{
-    ConfigSettingEntry, ConfigSettings, NoBuild, PackageNameSpecifier, SetupPyStrategy, Upgrade,
+    ConfigSettingEntry, ConfigSettings, NoBuild, PackageNameSpecifier, Reinstall, SetupPyStrategy,
+    Upgrade,
 };
 
 use crate::commands::{extra_name_with_clap_error, ExitStatus, ListFormat, VersionFormat};
@@ -305,7 +306,7 @@ struct PipCompileArgs {
     #[clap(long)]
     no_deps: bool,
 
-    #[clap(long, value_enum, default_value_t = ResolutionMode::default())]
+    #[clap(long, value_enum, default_value_t = ResolutionMode::default(), env = "UV_RESOLUTION")]
     resolution: ResolutionMode,
 
     #[clap(long, value_enum, default_value_t = PreReleaseMode::default(), conflicts_with = "pre", env = "UV_PRERELEASE")]
@@ -780,7 +781,7 @@ struct PipInstallArgs {
     #[clap(long, value_enum, default_value_t = install_wheel_rs::linker::LinkMode::default())]
     link_mode: install_wheel_rs::linker::LinkMode,
 
-    #[clap(long, value_enum, default_value_t = ResolutionMode::default())]
+    #[clap(long, value_enum, default_value_t = ResolutionMode::default(), env = "UV_RESOLUTION")]
     resolution: ResolutionMode,
 
     #[clap(long, value_enum, default_value_t = PreReleaseMode::default(), conflicts_with = "pre", env = "UV_PRERELEASE")]
@@ -1032,6 +1033,10 @@ struct PipUninstallArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 struct PipFreezeArgs {
+    /// Exclude any editable packages from output.
+    #[clap(long)]
+    exclude_editable: bool,
+
     /// Validate the virtual environment, to detect packages with missing dependencies or other
     /// issues.
     #[clap(long)]
@@ -1078,11 +1083,6 @@ struct PipFreezeArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 struct PipListArgs {
-    /// Validate the virtual environment, to detect packages with missing dependencies or other
-    /// issues.
-    #[clap(long)]
-    strict: bool,
-
     /// Only include editable projects.
     #[clap(short, long)]
     editable: bool,
@@ -1099,6 +1099,11 @@ struct PipListArgs {
     #[clap(long, value_enum, default_value_t = ListFormat::default())]
     format: ListFormat,
 
+    /// Validate the virtual environment, to detect packages with missing dependencies or other
+    /// issues.
+    #[clap(long)]
+    strict: bool,
+
     /// The Python interpreter for which packages should be listed.
     ///
     /// By default, `uv` lists packages in the currently activated virtual environment, or a virtual
@@ -1689,7 +1694,7 @@ async fn run() -> Result<ExitStatus> {
                 upgrade,
                 index_urls,
                 args.keyring_provider,
-                &reinstall,
+                reinstall,
                 args.link_mode,
                 args.compile,
                 setup_py,
@@ -1747,6 +1752,7 @@ async fn run() -> Result<ExitStatus> {
         Commands::Pip(PipNamespace {
             command: PipCommand::Freeze(args),
         }) => commands::pip_freeze(
+            args.exclude_editable,
             args.strict,
             args.python.as_deref(),
             args.system,
@@ -1756,11 +1762,11 @@ async fn run() -> Result<ExitStatus> {
         Commands::Pip(PipNamespace {
             command: PipCommand::List(args),
         }) => commands::pip_list(
-            args.strict,
             args.editable,
             args.exclude_editable,
             &args.exclude,
             &args.format,
+            args.strict,
             args.python.as_deref(),
             args.system,
             &cache,
